/*******************************************************************************
 * Copyright (c) 2011 INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 *     Wolfgang Kling - AtlanMod (EMN, INRIA, LINA)
 *     
 ******************************************************************************/

package org.eclipse.am3.moscript.atl.ext;

import org.eclipse.emf.common.util.URI;
import org.eclipse.gmt.am3.extractor.launchers.utils.ATLUtils;
import org.eclipse.m2m.atl.core.ATLCoreException;
import org.eclipse.m2m.atl.core.IInjector;
import org.eclipse.m2m.atl.core.IModel;
import org.eclipse.m2m.atl.core.IReferenceModel;
import org.eclipse.m2m.atl.core.emf.EMFModelFactory;
import org.eclipse.m2m.atl.core.service.CoreService;

/**
 * @author <a href="mailto:wolfgang.kling@inria.fr">Wolfgang Kling</a>
 */
public class MgMEnv {
	private static MgMEnv instance;
	private static  EMFModelFactory atl3modelFactory;
	
	private MgMEnv() {
		super();
		try {
			atl3modelFactory = (EMFModelFactory)CoreService.getModelFactory("EMF");
		} 
		catch (ATLCoreException e) {
			e.printStackTrace();
		}	 
	}

	public static MgMEnv getInstance() {
		if (MgMEnv.instance == null)
			MgMEnv.instance = new MgMEnv();
		return MgMEnv.instance;
	}
	
	public IReferenceModel loadReferenceModel(URI uri)	throws ATLCoreException {
		IReferenceModel refModel = atl3modelFactory.newReferenceModel();
		ATLUtils.getInstance().getInjector().inject(refModel, uri.toString());
		return refModel;
	}
	
	public IModel loadModel(URI uri, IReferenceModel referenceModel) throws ATLCoreException {
		// inject file according to the extention
		String extension = getExtension(uri);
		if (extension.equalsIgnoreCase("ecore") || extension.equalsIgnoreCase("xmi")) {
			IInjector injectorInstance =  CoreService.getInjector("EMF");
			
			IModel model = atl3modelFactory.newModel(referenceModel);
			injectorInstance.inject(model, uri.toString());
			return model;
		}
		else if (extension.equalsIgnoreCase("atl")) {
			IInjector injectorInstance = CoreService.getInjector("ATL");
			
			IModel model = atl3modelFactory.newModel(referenceModel);
			injectorInstance.inject(model, uri.toString());
			return model;
		}
		else if (extension.equalsIgnoreCase("xml")) {
			IInjector injectorInstance = CoreService.getInjector("XML");
			
			IModel model = atl3modelFactory.newModel(referenceModel);
			injectorInstance.inject(model, uri.toString());
			return model;
		}
		else {
			throw new ATLCoreException("Cannot find injector for ."	+ extension + " files. " +
					                   "File " + uri.toString() + " could not be injected");
		}
	}
	
	private String getExtension(URI uri) {
		String uriString = uri.toString();
		return uriString.substring(uriString.lastIndexOf('.') + 1);
	}
}
