/*******************************************************************************
 * Copyright (c) 2011 Obeo, Vrije Universiteit Brussel, INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Original Contributors: 
 *     William Piers - Obeo - initial API and implementation
 *     Dennis Wagelaar (Vrije Universiteit Brussel)
 * Adaptation to MoScript:
 *     Wolfgang Kling - AtlanMod (EMN, INRIA, LINA)
 *  
 ******************************************************************************/
package org.eclipse.am3.moscript.core.emf;

import java.io.IOException;
import java.net.URL;
import java.util.Collections;

import org.eclipse.am3.moscript.common.ResourceProvider;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.m2m.atl.core.ATLCoreException;
import org.eclipse.m2m.atl.core.IReferenceModel;
import org.eclipse.m2m.atl.core.ModelFactory;
import org.eclipse.m2m.atl.core.emf.EMFReferenceModel;
import org.eclipse.m2m.atl.core.emf.Messages;

/**
 * The EMF implementation of the {@link ModelFactory}.
 * 
 * @author <a href="mailto:william.piers@obeo.fr">William Piers</a>
 * @author <a href="mailto:dennis.wagelaar@vub.ac.be">Dennis Wagelaar</a>
 * @author <a href="mailto:wolfgang.kling@inria.fr">Wolfgang Kling</a>
 */
public class EMFModelFactory extends org.eclipse.m2m.atl.core.emf.EMFModelFactory {

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.m2m.atl.core.ModelFactory#getBuiltInResource(java.lang.String)
	 */
	@Override
	public IReferenceModel getBuiltInResource(String name) throws ATLCoreException {
		EMFReferenceModel model = new EMFReferenceModel(getMetametamodel(), this);
		URL url = ResourceProvider.getURL(name);
		if (url == null) {
			throw new ATLCoreException(Messages.getString("EMFModelFactory.BUILT_IN_NOT_FOUND", name)); //$NON-NLS-1$
		}
		Resource builtin = resourceSet.createResource(URI.createURI(name));
		try {
			builtin.load(url.openStream(), Collections.EMPTY_MAP);
		} catch (IOException e) {
			throw new ATLCoreException(Messages.getString("EMFModelFactory.BUILT_IN_NOT_FOUND", name), e); //$NON-NLS-1$
		}
		model.setResource(builtin);
		model.register();
		return model;
	}
}
