/*******************************************************************************
 * Copyright (c) 2011 Obeo, INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Original Contributors: 
 *     William Piers - Obeo - initial API and implementation
 * Adaptation to MoScript:
 *     Wolfgang Kling - AtlanMod (EMN, INRIA, LINA)
 *  
 ******************************************************************************/
package org.eclipse.gmt.am3.moscript.ui;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.logging.Level;

import org.eclipse.am3.moscript.atl.ext.MgMLibExtension;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.m2m.atl.adt.ui.common.WorkspaceFileDialog;
import org.eclipse.m2m.atl.common.ATLLaunchConstants;
import org.eclipse.m2m.atl.common.ATLLogger;
import org.eclipse.m2m.atl.core.ui.Messages;
import org.eclipse.m2m.atl.core.ui.launch.MainAtlTab;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Text;
import org.osgi.framework.Bundle;

/**
 * @author <a href="mailto:wolfgang.kling@inria.fr">Wolfgang Kling</a>
 */
public class MainMoScriptTab extends MainAtlTab {
	
	public void createControl(Composite parent) {
		scrollContainer = new ScrolledComposite(parent, SWT.H_SCROLL
				| SWT.V_SCROLL);
		scrollContainer.setExpandHorizontal(true);
		scrollContainer.setExpandVertical(true);

		rootContainer = new Composite(scrollContainer, SWT.NULL);
		rootContainer.setLayout(new GridLayout());
		scrollContainer.setContent(rootContainer);

		moduleGroup = new Group(rootContainer, SWT.NULL);
		moduleGroup.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		moduleGroup.setLayout(new GridLayout(3, false));
		moduleGroup.setText("MoScript Module"); //$NON-NLS-1$

		atlPathText = new Text(moduleGroup, SWT.SINGLE | SWT.BORDER);
		atlPathText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		atlPathText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				rebuild();
			}
		});

		final Button browseWorkspace = new Button(moduleGroup, SWT.RIGHT);
		browseWorkspace.setText(Messages.getString("MainAtlTab.WORKSPACE")); //$NON-NLS-1$

		atlPathText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updateLaunchConfigurationDialog();
			}
		});

		browseWorkspace.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent evt) {
				String path = editPath(IS_MODULE);
				if (path.trim().length() > 0) {
					atlPathText.setText(path);
				}
			}
		});

		librariesGroup = new Group(rootContainer, SWT.NULL);
		librariesGroup.setText(Messages.getString("MainAtlTab.LIBRARIES")); //$NON-NLS-1$
		librariesGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				false));
		librariesGroup.setLayout(new GridLayout(4, false));

		controlGroup = new Group(rootContainer, SWT.NULL);
		controlGroup.setText(Messages.getString("MainAtlTab.MODIFY")); //$NON-NLS-1$
		controlGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				false, 3, 1));
		controlGroup.setLayout(new GridLayout(7, false));

		final Button addLibBtn = new Button(controlGroup, SWT.NULL);
		addLibBtn.setText(Messages.getString("MainAtlTab.ADDLIB")); //$NON-NLS-1$


		sourceMetamodelsFromModule.add(MgMLibExtension.refModelKey);
		
		addLibBtn.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent evt) {
				InputDialog dlg = new InputDialog(getShell(), Messages
						.getString("MainAtlTab.ADDLIBRARY"), //$NON-NLS-1$
						Messages.getString("MainAtlTab.LIBRARYNAME"), //$NON-NLS-1$
						"LIB", //$NON-NLS-1$
						null);
				if (dlg.open() == InputDialog.CANCEL) {
					return;
				}
				String libName = dlg.getValue();
				if (!librariesFromModule.contains(libName)) {
					librariesFromModule.add(libName);
					removableLibraries.add(libName);
				}
				build();
				layout();
			}
		});

		setControl(scrollContainer);
	}

	protected void build() {
		buildLibraryControls(librariesFromModule);
	}

	private void layout() {
		librariesGroup.layout();

		rootContainer.layout();

		scrollContainer.setMinSize(rootContainer.computeSize(SWT.DEFAULT,
				SWT.DEFAULT));
		scrollContainer.layout();

		updateLaunchConfigurationDialog();
	}

	private void rebuild() {
		try {
			clear();
			build();
			initializeExtraWidgetsFrom(launchConfiguration);
			build();
			initializeWidgetValuesFrom(launchConfiguration);
			layout();
		} catch (CoreException ex) {
			ATLLogger.log(Level.SEVERE, ex.getLocalizedMessage(), ex);
		}
	}

	private void clear() {
		librariesFromModule.clear();
		removableLibraries.clear();

		librariesGroupWidgets.clear();

		disposeGroupChildren(librariesGroup);
	}

	private void initializeExtraWidgetsFrom(ILaunchConfiguration configuration)
			throws CoreException {
		if (configuration == null) {
			return;
		}

		Map<?, ?> savedLibraries = configuration.getAttribute(
				ATLLaunchConstants.LIBS, Collections.EMPTY_MAP);

		for (Iterator<?> i = savedLibraries.keySet().iterator(); i.hasNext();) {
			String libraryName = (String) i.next();
			if (!librariesFromModule.contains(libraryName)) {
				librariesFromModule.add(libraryName);
				removableLibraries.add(libraryName);
			}
		}
	}

	private void initializeWidgetValuesFrom(ILaunchConfiguration configuration)
			throws CoreException {
		if (configuration == null) {
			return;
		}

		Map<?, ?> savedLibraries = configuration.getAttribute(
				ATLLaunchConstants.LIBS, Collections.EMPTY_MAP);

		for (Iterator<?> i = savedLibraries.keySet().iterator(); i.hasNext();) {
			String libraryName = (String) i.next();
			Map<String, Object> widgets = librariesGroupWidgets
					.get(libraryName);
			if (widgets != null) {
				Text libraryLocation = (Text) widgets.get("location"); //$NON-NLS-1$
				libraryLocation.setText((String) savedLibraries
						.get(libraryName));
			}
		}
	}

	public void initializeFrom(ILaunchConfiguration configuration) {
		this.launchConfiguration = configuration;
		try {
			String fileName = configuration.getAttribute(
					ATLLaunchConstants.ATL_FILE_NAME, ""); //$NON-NLS-1$
			superimposedFromAdvancedTab = configuration.getAttribute(
					ATLLaunchConstants.SUPERIMPOSE, Collections.EMPTY_LIST);
			launcherNameFromAdvancedTab = configuration.getAttribute(
					ATLLaunchConstants.ATL_VM, ATLLaunchConstants.EMF_VM_NAME);

			if (!fileName.equals(atlPathText.getText())) {
				atlPathText.setText(fileName);
			}
		} catch (CoreException e) {
			ATLLogger.log(Level.SEVERE, e.getLocalizedMessage(), e);
		}
		rebuild();
	}

	public boolean isValid(ILaunchConfiguration launchConfig) {
		if (atlPathText.getText().equals("")) { //$NON-NLS-1$
			this.setErrorMessage("Please, give a MoScript file name"); //$NON-NLS-1$
			return false;
		}

		this.setErrorMessage(null);
		return canSaveGroupWidgets(librariesGroupWidgets)
				&& super.isValid(launchConfig);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#getImage()
	 */
	@Override
	public Image getImage() {
		return createImageDescriptor("/icons/MegaModel.gif").createImage();
	}

	public static ImageDescriptor createImageDescriptor(String path) {
		Bundle bundle = Platform.getBundle("org.eclipse.am3.moscript.ui");
		URL url = FileLocator.find(bundle, new Path(path), null);
		return ImageDescriptor.createFromURL(url);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
	 */
	public String getName() {
		return "MoScript configuration"; //$NON-NLS-1$
	}

	/**
	 * This method edit the path of the model selected. The path corresponding
	 * the a file in the workspace
	 * 
	 * @param type
	 */
	protected String editPath(final int type) {
		String ret = ""; //$NON-NLS-1$

		String[] extensions = null;
		if (type == IS_MODULE) {
			extensions = new String[] {"mscr",};
		} 
		else if (type == IS_LIBRARY) {
			extensions = new String[] { "asm" }; //$NON-NLS-1$
		}
		WorkspaceFileDialog dialog = new WorkspaceFileDialog(getShell(),
				extensions);
		dialog.open();
		Object result = dialog.getFirstResult();

		if ((result != null) && (result instanceof IFile)) {
			IFile currentFile = (IFile) result;
			ret = currentFile.getFullPath().toString();
		}

		return ret;
	}

	/**
	 * {@inheritDoc}
	 * 
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		Map<String, String> savedPaths = new HashMap<String, String>();
		Map<String, String> savedModelHandlers = new HashMap<String, String>();
		Map<String, String> savedLibraries = new HashMap<String, String>();
		Map<String, String> savedInput = new LinkedHashMap<String, String>();
		Map<String, String> savedOutput = new LinkedHashMap<String, String>();
		Map<String, String> savedType = new HashMap<String, String>();

		IPath path = ResourcesPlugin.getWorkspace().getRoot().getLocation();
		path = path.append(".am3");
		path = path.append("megamodel.xmi");
		
		//TODO get reference model path from extentions
		savedPaths.put(MgMLibExtension.refModelKey, "uri:http://www.eclipse.org/gmt/am3/2007/GMM4ATL"); //$NON-NLS-1$
		savedModelHandlers.put(MgMLibExtension.refModelKey, "EMF");
		savedPaths.put(MgMLibExtension.modelKey, "ext:" + path.toString());

		
		for (Iterator<String> it = librariesFromModule.iterator(); it.hasNext();) {
			String modelName = it.next();
			String location = ((Text)(librariesGroupWidgets.get(modelName)).get("location")).getText(); //$NON-NLS-1$
			savedLibraries.put(modelName, location);
		}

		savedInput.put(MgMLibExtension.modelKey, MgMLibExtension.refModelKey);
		savedType.put(MgMLibExtension.modelKey, ATLLaunchConstants.MODEL_INPUT);
		savedType.put(MgMLibExtension.refModelKey, ATLLaunchConstants.METAMODEL_INPUT);

		configuration.setAttribute(ATLLaunchConstants.ATL_FILE_NAME, atlPathText.getText());
		configuration.setAttribute(ATLLaunchConstants.PATH, savedPaths);
		configuration.setAttribute(ATLLaunchConstants.MODEL_HANDLER, savedModelHandlers);
		configuration.setAttribute(ATLLaunchConstants.LIBS, savedLibraries);
		configuration.setAttribute(ATLLaunchConstants.INPUT, savedInput);
		configuration.setAttribute(ATLLaunchConstants.OUTPUT, savedOutput);
		configuration.setAttribute(ATLLaunchConstants.MODEL_TYPE, savedType);
		configuration.setAttribute(ATLLaunchConstants.IS_REFINING, false);
		configuration.setAttribute(ATLLaunchConstants.ATL_COMPILER, atlcompiler);
		configuration.setAttribute(ATLLaunchConstants.ORDERED_INPUT, new ArrayList<String>(savedInput
				.keySet()));
		configuration.setAttribute(ATLLaunchConstants.ORDERED_OUTPUT, new ArrayList<String>(savedOutput
				.keySet()));
	}

}
