/*******************************************************************************
 * Copyright (c) 2011 Obeo, INRIA. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0 
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Original Contributors: 
 *     William Piers - Obeo - initial API and implementation
 * Adaptation to MoScript:
 *     Wolfgang Kling - AtlanMod (EMN, INRIA, LINA)
 *  
 ******************************************************************************/
package org.eclipse.gmt.am3.moscript.ui.builder;

import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
/**
 * @author <a href="mailto:william.piers@obeo.fr">William Piers</a>
 * @author <a href="mailto:wolfgang.kling@inria.fr">Wolfgang Kling</a>
 */
public class MoScriptBuilder extends IncrementalProjectBuilder {

	public static final String BUILDER_ID = "org.eclipse.am3.moscript.ui.moScriptBuilder";

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.internal.events.InternalBuilder#build(int,
	 *      java.util.Map, org.eclipse.core.runtime.IProgressMonitor)
	 */
	protected IProject[] build(int kind, Map args, IProgressMonitor monitor)
			throws CoreException {
		IWorkspaceRunnable wr = new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				fullBuild(monitor);
			}
		};
		run(wr, monitor);
		return null;
	}
	
	/**
	 * Process a full build.
	 * 
	 * @param monitor
	 *            the progress monitor
	 */
	protected void fullBuild(IProgressMonitor monitor) {
		try {
			IProject p = getProject();
			monitor.beginTask("Compiling MoScript files of project " + p.getName(), IProgressMonitor.UNKNOWN); //$NON-NLS-1$
			p.accept(new MoScriptBuildVisitor(monitor));
		} 
		catch (CoreException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Execute the given workspace runnable.
	 * 
	 * @param wr: the given workspace runnable
	 * @param monitor: the progress monitor
	 * @throws CoreException
	 */
	protected void run(IWorkspaceRunnable wr, IProgressMonitor monitor) throws CoreException {
		ResourcesPlugin.getWorkspace().run(wr, monitor);
	}	
}
